use chrono::{NaiveDate, NaiveDateTime};
use dotenv::dotenv;
use juunil::controllers::{images, posts, videos};
use raspa::request::{Request, RequestBase};
use raspa::selector::{Selector, SelectorBase};
use serde_json::Value;

const URL_BASE: &str = "https://syndication.twitter.com/srv/timeline-profile/screen-name/";
const DATE_FMT: &str = "%a %b %d %H:%M:%S %z %Y";

#[derive(Debug)]
struct PostInfo {
    id: i64,
    description: String,
    images: Vec<String>,
    videos: Vec<String>,
    datetime: NaiveDateTime,
}

fn build_timestamp_from_str(raw_date: String) -> NaiveDateTime {
    NaiveDateTime::parse_from_str(&raw_date, DATE_FMT).unwrap()
}

fn get_post_info(content: Value) -> PostInfo {
    let id = content["conversation_id_str"]
        .as_str()
        .unwrap()
        .parse::<i64>()
        .unwrap();

    let mut description = content["full_text"].as_str().unwrap().to_string();
    let datetime = build_timestamp_from_str(content["created_at"].as_str().unwrap().to_string());
    let mut images: Vec<String> = vec![];
    let mut videos: Vec<String> = vec![];

    let media_items = if content["extended_entities"]["media"].is_array() {
        content["extended_entities"]["media"].as_array().unwrap()
    } else {
        content["entities"]["media"].as_array().unwrap()
    };

    for media in media_items {
        let min_url = media["url"].as_str().unwrap();
        description = description.replace(min_url, "");

        let media_type = media["type"].as_str().unwrap();

        match media_type {
            "photo" => images.push(media["media_url_https"].as_str().unwrap().to_string()),
            "video" => {
                let mut bitrate = 0;
                let mut index = 0;
                for (i, variant) in media["video_info"]["variants"]
                    .as_array()
                    .unwrap()
                    .iter()
                    .enumerate()
                {
                    if variant["bitrate"].is_number() {
                        let temp_bitrate = variant["bitrate"].as_u64().unwrap();
                        if temp_bitrate > bitrate {
                            bitrate = temp_bitrate;
                            index = i;
                        }
                    }
                }
                videos.push(
                    media["video_info"]["variants"][index]["url"]
                        .as_str()
                        .unwrap()
                        .to_string(),
                );
            }
            _ => {}
        };
    }
    description = description.trim().to_string();

    for url in content["entities"]["urls"].as_array().unwrap() {
        let min_url = url["url"].as_str().unwrap();
        let max_url = url["expanded_url"].as_str().unwrap();
        description = description.replace(min_url, max_url);
    }

    PostInfo {
        id,
        description,
        images,
        videos,
        datetime,
    }
}

fn get_posts() -> Vec<PostInfo> {
    let tw_user: &str = &std::env::var("TW_USER")
        .expect("Could not load the environment variable \"TW_USER\", add it to your .env");
    let auth_token: &str = &std::env::var("AUTH_TOKEN")
        .expect("Could not load the environment variable \"AUTH_TOKEN\", add it to your .env");

    let resp = if cfg!(debug_assertions) {
        Selector::from_html(include_str!("../test2.html"))
    } else {
        let req = Request::new(format!("{}{}", URL_BASE, tw_user)).unwrap();
        let res = req.add_cookies(vec![("auth_token", auth_token)]).launch();
        Selector::from_html(res.html())
    };

    let raw_json = resp
        .xpath_once("//script[@id=\"__NEXT_DATA__\"]")
        .unwrap()
        .html();
    println!("{raw_json}");
    let raw_json: &[u8] = raw_json.as_ref();

    let data: Value = serde_json::from_slice(&raw_json[51..raw_json.len() - 9])
        .expect("The JSON could'nt be deserialized");
    let tws = data["props"]["pageProps"]["timeline"]["entries"]
        .as_array()
        .unwrap()
        .iter()
        // Remove all the RT statuses
        .filter(|x| {
            !x["content"]["tweet"]["full_text"]
                .as_str()
                .unwrap()
                .starts_with("RT")
        })
        // Remove all the reply tweets
        .filter(|x| !x["content"]["tweet"]["in_reply_to_status_id_str"].is_string())
        .collect::<Vec<_>>();

    let mut output = vec![];

    for tw in tws {
        let content = tw["content"]["tweet"].clone();
        let post_info = get_post_info(content);

        output.push(post_info);
    }
    output
}

fn main() {
    dotenv().ok();

    let latest_posts = get_posts();
    println!("{latest_posts:#?}");

    for post in latest_posts {
        posts::add_post(post.id, post.description, post.datetime);
        images::add_images(post.id, post.images);
        videos::add_videos(post.id, post.videos);
    }
}
