use chrono::prelude::Utc;
use chrono::NaiveDateTime;
use diesel::sqlite::SqliteConnection;
use diesel::Connection;
use diesel::ExpressionMethods;
use diesel::QueryDsl;
use diesel::RunQueryDsl;
use diesel::SelectableHelper;

use std::env;

pub fn establish_connection() -> SqliteConnection {
    let database_url = env::var("DATABASE_URL").expect("DATABASE_URL env var not seted");

    SqliteConnection::establish(&database_url)
        .expect(&format!("Error connecting to {}", database_url))
}

pub mod posts {
    use crate::controllers::*;
    use crate::models::{Post, PostWithMedia};
    use crate::schema::post;

    pub fn add_post(post_id: i64, description: String, posted_date: NaiveDateTime) {
        let mut conn = establish_connection();

        let crawled_date = Utc::now().naive_utc();

        let new_post = Post {
            id: post_id,
            description,
            posted_date,
            crawled_date,
        };

        diesel::insert_into(post::table)
            .values(&new_post)
            .execute(&mut conn)
            .expect("Error adding the post");
    }

    pub fn get_post(post_id: i64) -> Post {
        use crate::schema::post::dsl::*;

        let mut conn = establish_connection();

        post.find(post_id)
            .first(&mut conn)
            .expect("Error getting the post")
    }

    pub fn get_post_with_media(post_id: i64) -> PostWithMedia {
        let images = images::get_images_strings(post_id);
        let videos = videos::get_videos_strings(post_id);
        let post = get_post(post_id);

        PostWithMedia {
            id: post.id,
            description: post.description,
            posted_date: post.posted_date,
            crawled_date: post.crawled_date,
            images,
            videos,
        }
    }

    pub fn get_all_posts_with_media() -> Vec<PostWithMedia> {
        use crate::schema::post::dsl::*;

        let mut conn = establish_connection();

        let posts = post
            .order(crawled_date.asc())
            .load(&mut conn)
            .expect("Error getting the posts");

        let mut output = vec![];

        for post_without_media in posts {
            let mut new_post = PostWithMedia::from_post(&post_without_media);
            let images = images::get_images_strings(new_post.id);
            let videos = videos::get_videos_strings(new_post.id);
            new_post.images = images;
            new_post.videos = videos;
            output.push(new_post);
        }
        output
    }
}

pub mod images {
    use crate::controllers::*;
    use crate::models::{Image, NewImage};
    use crate::schema::image;

    pub fn add_image(post_id: i64, url: String) {
        let mut conn = establish_connection();

        let new_image = NewImage { post_id, url };

        diesel::insert_into(image::table)
            .values(&new_image)
            .execute(&mut conn)
            .expect("Error adding the image");
    }

    pub fn add_images(post_id: i64, urls: Vec<String>) {
        for url in urls {
            add_image(post_id, url);
        }
    }

    pub fn get_images(post: i64) -> Vec<Image> {
        use crate::schema::image::dsl::*;
        let mut conn = establish_connection();

        image
            .filter(post_id.eq(post))
            .select(Image::as_select())
            .load(&mut conn)
            .expect("Error getting the images")
    }

    pub fn get_images_strings(post: i64) -> Vec<String> {
        get_images(post)
            .iter()
            .map(|image| image.url.clone())
            .collect::<Vec<String>>()
    }
}

pub mod videos {
    use crate::controllers::*;
    use crate::models::{NewVideo, Video};
    use crate::schema::video;

    pub fn add_video(post_id: i64, url: String) {
        let mut conn = establish_connection();

        let new_video = NewVideo { post_id, url };

        diesel::insert_into(video::table)
            .values(&new_video)
            .execute(&mut conn)
            .expect("Error adding the video");
    }

    pub fn add_videos(post_id: i64, urls: Vec<String>) {
        for url in urls {
            add_video(post_id, url);
        }
    }

    pub fn get_videos(post: i64) -> Vec<Video> {
        use crate::schema::video::dsl::*;
        let mut conn = establish_connection();

        video
            .filter(post_id.eq(post))
            .select(Video::as_select())
            .load(&mut conn)
            .expect("Error getting the videos")
    }

    pub fn get_videos_strings(post: i64) -> Vec<String> {
        get_videos(post)
            .iter()
            .map(|video| video.url.clone())
            .collect::<Vec<String>>()
    }
}
